@@ -1,4 +1,5 @@
 from django.conf.urls.defaults import patterns, url
+from django.contrib.auth.decorators import login_required
 
 from oscar.apps.basket.views import (BasketView, SavedView,
     VoucherAddView, BasketAddView, VoucherRemoveView)
@@ -19,7 +20,7 @@ class BasketApplication(Application):
             url(r'^add/$', self.add_view.as_view(), name='add'),    
             url(r'^vouchers/add/$', self.add_voucher_view.as_view(), name='vouchers-add'),
             url(r'^vouchers/(?P<pk>\d+)/remove/$', self.remove_voucher_view.as_view(), name='vouchers-remove'),     
-            url(r'^saved/$', self.saved_view.as_view(), name='saved'),
+            url(r'^saved/$', login_required(self.saved_view.as_view()), name='saved'),
         )
         return self.post_process_urls(urlpatterns)
 
