@@ -3,33 +3,37 @@ import math
 import datetime
 
 from django.core import exceptions
+from django.template.defaultfilters import slugify
 from django.db import models
 from django.utils.translation import ugettext as _
 from django.core.exceptions import ValidationError
+from django.core.urlresolvers import reverse
 from django.conf import settings
 
 from oscar.apps.offer.managers import ActiveOfferManager
 from oscar.models.fields import PositiveDecimalField, ExtendedURLField
 
-SITE, VOUCHER, USER, SESSION = ("Site", "Voucher", "User", "Session")
-
 
 class ConditionalOffer(models.Model):
     """
     A conditional offer (eg buy 1, get 10% off)
     """
-    name = models.CharField(max_length=128)
+    name = models.CharField(max_length=128, unique=True, 
+                            help_text="""This is displayed within the customer's
+                            basket""")
+    slug = models.SlugField(max_length=128, unique=True, null=True)
     description = models.TextField(blank=True, null=True)
 
     # Offers come in a few different types:
-    # (a) Offers that are available to all customers on the site.  Eg a
+    # (a) Offers that are available to all customers on the site.  Eg a 
     #     3-for-2 offer.
     # (b) Offers that are linked to a voucher, and only become available once
     #     that voucher has been applied to the basket
     # (c) Offers that are linked to a user.  Eg, all students get 10% off.  The code
     #     to apply this offer needs to be coded
-    # (d) Session offers - these are temporarily available to a user after some trigger
-    #     event.  Eg, users coming from some affiliate site get 10% off.
+    # (d) Session offers - these are temporarily available to a user after some trigger 
+    #     event.  Eg, users coming from some affiliate site get 10% off.     
+    SITE, VOUCHER, USER, SESSION = ("Site", "Voucher", "User", "Session")
     TYPE_CHOICES = (
         (SITE, "Site offer - available to all users"),
         (VOUCHER, "Voucher offer - only available after entering the appropriate voucher code"),
@@ -42,17 +46,20 @@ class ConditionalOffer(models.Model):
     benefit = models.ForeignKey('offer.Benefit')
 
     # Range of availability.  Note that if this is a voucher offer, then these
-    # dates are ignored and only the dates from the voucher are used to determine
+    # dates are ignored and only the dates from the voucher are used to determine 
     # availability.
     start_date = models.DateField(blank=True, null=True)
-    end_date = models.DateField(blank=True, null=True)
+    end_date = models.DateField(blank=True, null=True,
+                                help_text="""Offers are not active on their end
+                                date, only the days preceding""")
 
     # Some complicated situations require offers to be applied in a set order.
     priority = models.IntegerField(default=0, help_text="The highest priority offers are applied first")
 
     # We track some information on usage
     total_discount = models.DecimalField(decimal_places=2, max_digits=12, default=Decimal('0.00'))
-
+    num_orders = models.PositiveIntegerField(default=0)
+    
     date_created = models.DateTimeField(auto_now_add=True)
 
     objects = models.Manager()
@@ -66,42 +73,58 @@ class ConditionalOffer(models.Model):
     class Meta:
         ordering = ['-priority']
 
+    def save(self, *args, **kwargs):
+        if not self.slug:
+            self.slug = slugify(self.name)
+        return super(ConditionalOffer, self).save(*args, **kwargs)
+
+    def get_absolute_url(self):
+        return reverse('offer:detail', kwargs={'slug': self.slug})
+        
     def __unicode__(self):
-        return self.name
+        return self.name    
 
     def clean(self):
         if self.start_date and self.end_date and self.start_date > self.end_date:
             raise exceptions.ValidationError('End date should be later than start date')
-
+        
     def is_active(self, test_date=None):
         if not test_date:
             test_date = datetime.date.today()
         return self.start_date <= test_date and test_date < self.end_date
-
+    
     def is_condition_satisfied(self, basket):
         return self._proxy_condition().is_satisfied(basket)
 
+    def is_condition_partially_satisfied(self, basket):
+        return self._proxy_condition().is_partially_satisfied(basket)
+
+    def get_upsell_message(self, basket):
+        return self._proxy_condition().get_upsell_message(basket)
+        
     def apply_benefit(self, basket):
-        u"""
+        """
         Applies the benefit to the given basket and returns the discount.
         """
         if not self.is_condition_satisfied(basket):
             return Decimal('0.00')
         return self._proxy_benefit().apply(basket, self._proxy_condition())
-
+        
     def set_voucher(self, voucher):
         self._voucher = voucher
-
+        
     def get_voucher(self):
-        return self._voucher
-
+        return self._voucher        
+        
     def _proxy_condition(self):
-        u"""
+        """
         Returns the appropriate proxy model for the condition
         """
         field_dict = dict(self.condition.__dict__)
         if '_state' in field_dict:
             del field_dict['_state']
+        if '_range_cache' in field_dict:
+            del field_dict['_range_cache']
         if self.condition.type == self.condition.COUNT:
             return CountCondition(**field_dict)
         elif self.condition.type == self.condition.VALUE:
@@ -109,9 +132,9 @@ class ConditionalOffer(models.Model):
         elif self.condition.type == self.condition.COVERAGE:
             return CoverageCondition(**field_dict)
         return self.condition
-
+    
     def _proxy_benefit(self):
-        u"""
+        """
         Returns the appropriate proxy model for the condition
         """
         field_dict = dict(self.benefit.__dict__)
@@ -127,6 +150,11 @@ class ConditionalOffer(models.Model):
             return FixedPriceBenefit(**field_dict)
         return self.benefit
 
+    def record_usage(self, discount):
+        self.num_orders += 1
+        self.total_discount += discount
+        self.save()
+        
 
 class Condition(models.Model):
     COUNT, VALUE, COVERAGE = ("Count", "Value", "Coverage")
@@ -146,9 +174,11 @@ class Condition(models.Model):
             return u"Basket includes %d distinct products from %s" % (self.value, unicode(self.range).lower())
         return u"Basket includes %d value from %s" % (self.value, unicode(self.range).lower())
 
+    description = __unicode__
+    
     def consume_items(self, basket, lines=None):
         return ()
-
+    
     def is_satisfied(self, basket):
         """
         Determines whether a given basket meets this condition.  This is
@@ -157,6 +187,17 @@ class Condition(models.Model):
         """
         return False
 
+    def is_partially_satisfied(self, basket):
+        """
+        Determine if the basket partially meets the condition.  This is useful
+        for up-selling messages to entice customers to buy something more in
+        order to qualify for an offer.
+        """
+        return False
+
+    def get_upsell_message(self, basket):
+        return None
+    
 
 class Benefit(models.Model):
     PERCENTAGE, FIXED, MULTIBUY, FIXED_PRICE = ("Percentage", "Absolute", "Multibuy", "Fixed price")
@@ -173,11 +214,11 @@ class Benefit(models.Model):
 
     price_field = 'price_incl_tax'
 
-    # If this is not set, then there is no upper limit on how many products
+    # If this is not set, then there is no upper limit on how many products 
     # can be discounted by this benefit.
     max_affected_items = models.PositiveIntegerField(blank=True, null=True, help_text="""Set this
         to prevent the discount consuming all items within the range that are in the basket.""")
-
+    
     def __unicode__(self):
         if self.type == self.PERCENTAGE:
             desc = u"%s%% discount on %s" % (self.value, unicode(self.range).lower())
@@ -193,9 +234,11 @@ class Benefit(models.Model):
             desc += u" (max %d items)" % self.max_affected_items
         return desc
 
+    description = __unicode__
+    
     def apply(self, basket, condition=None):
         return Decimal('0.00')
-
+    
     def clean(self):
         if self.value is None:
             if not self.type:
@@ -232,15 +275,17 @@ class Range(models.Model):
     includes_all_products = models.BooleanField(default=False)
     included_products = models.ManyToManyField('catalogue.Product', related_name='includes', blank=True)
     excluded_products = models.ManyToManyField('catalogue.Product', related_name='excludes', blank=True)
-    classes = models.ManyToManyField('catalogue.ProductClass', blank=True)
+    classes = models.ManyToManyField('catalogue.ProductClass', related_name='classes', blank=True)
     included_categories = models.ManyToManyField('catalogue.Category', related_name='includes', blank=True)
-
+    date_created = models.DateTimeField(auto_now_add=True)
+    
     __included_product_ids = None
     __excluded_product_ids = None
+    __class_ids = None
 
     def __unicode__(self):
-        return self.name
-
+        return self.name    
+        
     def contains_product(self, product):
         """
         Check whether the passed product is part of this range
@@ -257,38 +302,38 @@ class Range(models.Model):
         if self.includes_all_products:
             return True
         if product.product_class_id in self._class_ids():
-            return True
+            return True   
         included_product_ids = self._included_product_ids()
         if product.id in included_product_ids:
             return True
-        test_categories = self.included_categories.all()
+        test_categories = self.included_categories.all() 
         if test_categories:
             for category in product.categories.all():
                 for test_category in test_categories:
                     if category == test_category or category.is_descendant_of(test_category):
                         return True
         return False
-
+    
     def _included_product_ids(self):
         if None == self.__included_product_ids:
             self.__included_product_ids = [row['id'] for row in self.included_products.values('id')]
         return self.__included_product_ids
-
+    
     def _excluded_product_ids(self):
         if None == self.__excluded_product_ids:
             self.__excluded_product_ids = [row['id'] for row in self.excluded_products.values('id')]
         return self.__excluded_product_ids
-
+    
     def _class_ids(self):
-        if not hasattr(self, '__class_ids'):
-            # This is deliberately disabled as it causes a nasty problem in the
-            # Landmark project which we've been unable to reproduce.  Will
-            # re-enable when the problem is resolved.  Apologies for any
-            # inconvenience.
-            #self.__class_ids = [row['id'] for row in self.klasses.values('id')]
-            self.__class_ids = []
+        if None == self.__class_ids:
+            self.__class_ids = [row['id'] for row in self.classes.values('id')]
         return self.__class_ids
 
+    def num_products(self):
+        if self.includes_all_products:
+            return None
+        return self.included_products.all().count()
+        
 
 class CountCondition(Condition):
     """
@@ -310,6 +355,26 @@ class CountCondition(Condition):
                 return True
         return False
 
+    def _get_num_matches(self, basket):
+        if hasattr(self, '_num_matches'):
+            return getattr(self, '_num_matches')
+        num_matches = 0
+        for line in basket.all_lines():
+            if self.range.contains_product(line.product) and line.quantity_without_discount > 0:
+                num_matches += line.quantity_without_discount
+        self._num_matches = num_matches
+        return num_matches
+
+    def is_partially_satisfied(self, basket):
+        num_matches = self._get_num_matches(basket)
+        return 0 < num_matches < self.value
+
+    def get_upsell_message(self, basket):
+        num_matches = self._get_num_matches(basket)
+        delta = self.value - num_matches
+        return 'Buy %d more product%s from %s' % (delta,
+                                                  's' if delta > 1 else '', self.range)
+    
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
@@ -327,11 +392,11 @@ class CountCondition(Condition):
             if len(consumed_products) == value:
                 break
         return consumed_products
-
-
+        
+        
 class CoverageCondition(Condition):
     """
-    An offer condition dependent on the NUMBER of matching items from the basket.
+    An offer condition dependent on the number of DISTINCT matching items from the basket.
     """
     class Meta:
         proxy = True
@@ -351,6 +416,24 @@ class CoverageCondition(Condition):
                 return True
         return False
 
+    def _get_num_covered_products(self, basket):
+        covered_ids = []
+        for line in basket.all_lines():
+            if not line.is_available_for_discount:
+                continue
+            product = line.product
+            if self.range.contains_product(product) and product.id not in covered_ids:
+                covered_ids.append(product.id)
+        return len(covered_ids)
+
+    def get_upsell_message(self, basket):
+        delta = self.value - self._get_num_covered_products(basket)
+        return 'Buy %d more product%s from %s' % (delta,
+                                                  's' if delta > 1 else '', self.range)
+
+    def is_partially_satisfied(self, basket):
+        return 0 < self._get_num_covered_products(basket) < self.value
+    
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
@@ -368,7 +451,7 @@ class CoverageCondition(Condition):
             if len(consumed_products) >= value:
                 break
         return consumed_products
-
+    
     def get_value_of_satisfying_items(self, basket):
         covered_ids = []
         value = Decimal('0.00')
@@ -379,8 +462,8 @@ class CoverageCondition(Condition):
             if len(covered_ids) >= self.value:
                 return value
         return value
-
-
+        
+        
 class ValueCondition(Condition):
     """
     An offer condition dependent on the VALUE of matching items from the basket.
@@ -402,11 +485,31 @@ class ValueCondition(Condition):
                 return True
         return False
 
+    def _get_value_of_matches(self, basket):
+        if hasattr(self, '_value_of_matches'):
+            return getattr(self, '_value_of_matches')
+        value_of_matches = Decimal('0.00')
+        for line in basket.all_lines():
+            product = line.product
+            if self.range.contains_product(product) and product.has_stockrecord and line.quantity_without_discount > 0:
+                price = getattr(product.stockrecord, self.price_field)
+                value_of_matches += price * int(line.quantity_without_discount)
+        self._value_of_matches = value_of_matches
+        return value_of_matches
+
+    def is_partially_satisfied(self, basket):
+        value_of_matches = self._get_value_of_matches(basket)
+        return Decimal('0.00') < value_of_matches < self.value
+
+    def get_upsell_message(self, basket):
+        value_of_matches = self._get_value_of_matches(basket)
+        return 'Spend %s more from %s' % (value_of_matches, self.range)
+    
     def consume_items(self, basket, lines=None, value=None):
         """
         Marks items within the basket lines as consumed so they
         can't be reused in other offers.
-
+        
         We allow lines to be passed in as sometimes we want them sorted
         in a specific order.
         """
@@ -421,11 +524,11 @@ class ValueCondition(Condition):
                 if not price:
                     continue
                 quantity_to_consume = min(line.quantity_without_discount,
-                                          int(((value - value_of_matches) / price).quantize(Decimal(1),
+                                          int(((value - value_of_matches)/price).quantize(Decimal(1),
                                                                                               ROUND_UP)))
                 value_of_matches += price * quantity_to_consume
                 line.consume(quantity_to_consume)
-                consumed_products.extend((line.product,) * int(quantity_to_consume))
+                consumed_products.extend((line.product,)*int(quantity_to_consume))
             if value_of_matches >= value:
                 break
         return consumed_products
@@ -446,20 +549,20 @@ class PercentageDiscountBenefit(Benefit):
         discount = Decimal('0.00')
         affected_items = 0
         max_affected_items = self._effective_max_affected_items()
-
+        
         for line in basket.all_lines():
             if affected_items >= max_affected_items:
                 break
             product = line.product
             if self.range.contains_product(product) and product.has_stockrecord:
                 price = getattr(product.stockrecord, self.price_field)
-                quantity = min(line.quantity_without_discount,
+                quantity = min(line.quantity_without_discount, 
                                max_affected_items - affected_items)
-                line_discount = self.round(self.value / 100 * price * int(quantity))
+                line_discount = self.round(self.value/100 * price * int(quantity))
                 line.discount(line_discount, quantity)
                 affected_items += quantity
                 discount += line_discount
-
+                
         if discount > 0 and condition:
             condition.consume_items(basket)
         return discount
@@ -477,7 +580,7 @@ class AbsoluteDiscountBenefit(Benefit):
         discount = Decimal('0.00')
         affected_items = 0
         max_affected_items = self._effective_max_affected_items()
-
+        
         for line in basket.all_lines():
             if affected_items >= max_affected_items:
                 break
@@ -488,34 +591,32 @@ class AbsoluteDiscountBenefit(Benefit):
                     # Avoid zero price products
                     continue
                 remaining_discount = self.value - discount
-                quantity_affected = int(min(line.quantity_without_discount,
+                quantity_affected = int(min(line.quantity_without_discount, 
                                         max_affected_items - affected_items,
                                         math.ceil(remaining_discount / price)))
-
+                
                 # Update line with discounts
                 line_discount = self.round(min(remaining_discount, quantity_affected * price))
-                if condition:
-                    # Pass zero as quantity to avoid double consumption
-                    line.discount(line_discount, 0)
-                else:
-                    line.discount(line_discount, quantity_affected)
+                line.discount(line_discount, quantity_affected)
+                
                 # Update loop vars
                 affected_items += quantity_affected
                 remaining_discount -= line_discount
                 discount += line_discount
         if discount > 0 and condition:
             condition.consume_items(basket)
+            
         return discount
 
 
 class FixedPriceBenefit(Benefit):
     """
-    An offer benefit that gives the items in the condition for a
+    An offer benefit that gives the items in the condition for a 
     fixed price.  This is useful for "bundle" offers.
-
+    
     Note that we ignore the benefit range here and only give a fixed price
     for the products in the condition range.
-
+    
     The condition should be a count condition
     """
     class Meta:
@@ -529,22 +630,22 @@ class FixedPriceBenefit(Benefit):
         for line in basket.all_lines():
             product = line.product
             if condition.range.contains_product(product) and line.quantity_without_discount > 0:
-                # Line is available - determine quantity to consume and
+                # Line is available - determine quantity to consume and 
                 # record the total of the consumed products
                 if isinstance(condition, CoverageCondition):
                     quantity = 1
                 else:
                     quantity = min(line.quantity_without_discount, num_permitted - num_covered)
                 num_covered += quantity
-                product_total += quantity * line.unit_price_incl_tax
+                product_total += quantity*line.unit_price_incl_tax
                 covered_lines.append((line, quantity))
             if num_covered >= num_permitted:
                 break
         discount = max(product_total - self.value, Decimal('0.00'))
-
+        
         if not discount:
             return discount
-
+        
         # Apply discount weighted by original value of line
         discount_applied = Decimal('0.00')
         last_line = covered_lines[-1][0]
@@ -557,13 +658,7 @@ class FixedPriceBenefit(Benefit):
                 line_discount = self.round(discount * (line.unit_price_incl_tax * quantity) / product_total)
             line.discount(line_discount, quantity)
             discount_applied += line_discount
-        return discount
-
-        # Apply discount weighted by original value of line
-        for line, quantity in covered_lines:
-            line_discount = self.round(discount * (line.unit_price_incl_tax * quantity) / product_total)
-            line.discount(line_discount.quantize(Decimal('.01')), quantity)
-        return discount
+        return discount 
 
 
 class MultibuyDiscountBenefit(Benefit):
