@@ -1,10 +1,12 @@
 from django.dispatch import receiver
 from django.db.models.signals import m2m_changed, post_save
+from django.db.models import get_model
 
-from oscar.core.loading import import_module
 from oscar.apps.basket.abstract_models import AbstractBasket
-import_module('voucher.models', ['Voucher'], locals())
-import_module('order.models', ['OrderDiscount'], locals())
+Voucher = get_model('voucher', 'Voucher')
+OrderDiscount = get_model('order', 'OrderDiscount')
+ConditionalOffer = get_model('offer', 'ConditionalOffer')
+
 
 @receiver(m2m_changed)
 def receive_basket_voucher_change(sender, **kwargs):
@@ -15,6 +17,7 @@ def receive_basket_voucher_change(sender, **kwargs):
         voucher.num_basket_additions += 1
         voucher.save()
 
+
 @receiver(post_save, sender=OrderDiscount)        
 def receive_order_discount_save(sender, instance, **kwargs):
     # Record the amount of discount against the appropriate offers
@@ -23,9 +26,5 @@ def receive_order_discount_save(sender, instance, **kwargs):
     if discount.voucher:
         discount.voucher.total_discount += discount.amount
         discount.voucher.save()
-    discount.offer.total_discount += discount.amount
-    discount.offer.save()
-    
-    
-        
-    
+    if discount.offer:
+        discount.offer.record_usage(discount.amount)
