@@ -16,12 +16,12 @@ Item = get_model('product', 'Item')
 
 
 class LinkedPromotion(models.Model):
-    
+
     # We use generic foreign key to link to a promotion model
     content_type = models.ForeignKey(ContentType)
     object_id = models.PositiveIntegerField()
     content_object = generic.GenericForeignKey('content_type', 'object_id')
-    
+
     position = models.CharField(_("Position"), max_length=100, help_text="Position on page")
     display_order = models.PositiveIntegerField(default=0)
     clicks = models.PositiveIntegerField(default=0)
@@ -44,11 +44,11 @@ class PagePromotion(LinkedPromotion):
 
     def __unicode__(self):
         return u"%s on %s" % (self.content_object, self.page_url)
-    
+
     def get_link(self):
         return reverse('promotions:page-click', kwargs={'page_promotion_id': self.id})
-        
-    
+
+
 class KeywordPromotion(LinkedPromotion):
     """
     A promotion linked to a specific keyword.
@@ -58,7 +58,7 @@ class KeywordPromotion(LinkedPromotion):
     """
 
     keyword = models.CharField(_("Keyword"), max_length=200)
-    
+
     # We allow an additional filter which will let search query matches
     # be restricted to different parts of the site.
     filter = models.CharField(_("Filter"), max_length=200, blank=True, null=True)
@@ -78,36 +78,43 @@ class AbstractPromotion(models.Model):
     _type = 'Promotion'
     keywords = generic.GenericRelation(KeywordPromotion)
     pages = generic.GenericRelation(PagePromotion)
-    
+
     class Meta:
         abstract = True
-    
+
     @property
     def type(self):
         return _(self._type)
-    
+
+    @classmethod
+    def classname(cls):
+        return cls.__name__.lower()
+
+    @property
+    def code(self):
+        return self.__class__.__name__.lower()
+
     def template_name(self):
         """
-        Returns the template to use to render this
-        promotion.
+        Returns the template to use to render this promotion.
         """
-        return 'promotions/%s.html' % self.__class__.__name__.lower()
-    
-    def template_context(self, *args, **kwargs):
+        return 'promotions/%s.html' % self.code
+
+    def template_context(self, request):
         return {}
-    
+
     @property
     def content_type(self):
         return ContentType.objects.get_for_model(self)
-    
+
     @property
     def num_times_used(self):
         ctype = self.content_type
         page_count = PagePromotion.objects.filter(content_type=ctype,
                                                   object_id=self.id).count()
         keyword_count = KeywordPromotion.objects.filter(content_type=ctype,
-                                                        object_id=self.id).count()   
-        return page_count + keyword_count                                       
+                                                        object_id=self.id).count()
+        return page_count + keyword_count
 
 
 class RawHTML(AbstractPromotion):
@@ -116,16 +123,20 @@ class RawHTML(AbstractPromotion):
     """
     _type = 'Raw HTML'
     name = models.CharField(_("Name"), max_length=128)
-    
+
     # Used to determine how to render the promotion (eg
-    # if a different width container is required).
-    display_type = models.CharField(_("Display type"), max_length=128, blank=True, null=True)
+    # if a different width container is required).  This isn't always
+    # required.
+    display_type = models.CharField(
+        _("Display type"), max_length=128,
+        blank=True, null=True,
+        help_text="This can be used to have different types of HTML blocks (eg different widths)")
     body = models.TextField(_("HTML"))
     date_created = models.DateTimeField(auto_now_add=True)
 
     class Meta:
         verbose_name_plural = 'Raw HTML'
-        
+
     def __unicode__(self):
         return self.name
 
@@ -143,11 +154,11 @@ class Image(AbstractPromotion):
         where this promotion links to""")
     image = models.ImageField(upload_to=settings.OSCAR_PROMOTION_FOLDER)
     date_created = models.DateTimeField(auto_now_add=True)
-    
+
     def __unicode__(self):
         return self.name
-    
-    
+
+
 class MultiImage(AbstractPromotion):
     """
     A multi-image promotion is simply a collection of image promotions
@@ -158,7 +169,7 @@ class MultiImage(AbstractPromotion):
     name = models.CharField(_("Name"), max_length=128)
     images = models.ManyToManyField('promotions.Image', null=True, blank=True)
     date_created = models.DateTimeField(auto_now_add=True)
-    
+
     def __unicode__(self):
         return self.name
 
@@ -169,10 +180,13 @@ class SingleProduct(AbstractPromotion):
     product = models.ForeignKey('catalogue.Product')
     description = models.TextField(_("Description"), null=True, blank=True)
     date_created = models.DateTimeField(auto_now_add=True)
-    
+
     def __unicode__(self):
         return self.name
-    
+
+    def template_context(self, request):
+        return {'product': self.product}
+
 
 class AbstractProductList(AbstractPromotion):
     """
@@ -184,14 +198,17 @@ class AbstractProductList(AbstractPromotion):
     link_url = ExtendedURLField(blank=True, null=True)
     link_text = models.CharField(_("Link text"), max_length=255, blank=True, null=True)
     date_created = models.DateTimeField(auto_now_add=True)
-    
+
     class Meta:
         abstract = True
-        
+
     def __unicode__(self):
         return self.name
-        
- 
+
+    def template_context(self, request):
+        return {'products': self.get_products()}
+
+
 class HandPickedProductList(AbstractProductList):
     """
     A hand-picked product list is a list of manually selected
@@ -200,22 +217,25 @@ class HandPickedProductList(AbstractProductList):
     _type = 'Product list'
     products = models.ManyToManyField('catalogue.Product', through='OrderedProduct', blank=True, null=True)
 
-    def get_products(self):
+    def get_queryset(self):
         return self.products.all().order_by('%s.display_order' % OrderedProduct._meta.db_table)
-    
+
+    def get_products(self):
+        return self.get_queryset()
+
 
 class OrderedProduct(models.Model):
-    
+
     list = models.ForeignKey('promotions.HandPickedProductList')
     product = models.ForeignKey('catalogue.Product')
     display_order = models.PositiveIntegerField(default=0)
-    
+
     class Meta:
         ordering = ('display_order',)
-        
-        
+
+
 class AutomaticProductList(AbstractProductList):
-    
+
     _type = 'Auto-product list'
     BESTSELLING, RECENTLY_ADDED = ('Bestselling', 'RecentlyAdded')
     METHOD_CHOICES = (
@@ -223,12 +243,21 @@ class AutomaticProductList(AbstractProductList):
         (RECENTLY_ADDED, _("Recently added products")),
     )
     method = models.CharField(max_length=128, choices=METHOD_CHOICES)
-    num_products = models.PositiveSmallIntegerField(default=4)  
-    
-    def get_products(self):
+    num_products = models.PositiveSmallIntegerField(default=4)
+
+    def get_queryset(self):
         if self.method == self.BESTSELLING:
-            return Product.objects.all().order_by('-score')[:self.num_products]
-        return Product.objects.all().order_by('-date_created')[:self.num_products]
+            return (Product.browsable.all()
+                    .select_related('stockrecord__partner')
+                    .prefetch_related('variants', 'images')
+                    .order_by('-score'))
+        return (Product.browsable.all()
+                .select_related('stockrecord__partner')
+                .prefetch_related('variants', 'images')
+                .order_by('-date_created'))
+
+    def get_products(self):
+        return self.get_queryset()[:self.num_products]
 
 
 class OrderedProductList(HandPickedProductList):
