@@ -1,3 +1,6 @@
+OSCAR_SHOP_NAME = 'Oscar'
+OSCAR_SHOP_TAGLINE = 'Domain-driven e-Commerce for Django'
+
 # Basket settings
 OSCAR_BASKET_COOKIE_LIFETIME = 7*24*60*60
 OSCAR_BASKET_COOKIE_OPEN = 'oscar_open_basket'
@@ -9,10 +12,15 @@ OSCAR_DEFAULT_CURRENCY = 'GBP'
 # Max number of products to keep on the user's history
 OSCAR_RECENTLY_VIEWED_PRODUCTS = 4
 
-# Image paths
+# Paths
 OSCAR_IMAGE_FOLDER = 'images/products/%Y/%m/'
 OSCAR_PROMOTION_FOLDER = 'images/promotions/'
 
+# Copy this image from oscar/static/img to your MEDIA_ROOT folder.
+# It needs to be there so Sorl can resize it.
+OSCAR_MISSING_IMAGE_URL = 'image_not_found.jpg'
+OSCAR_UPLOAD_ROOT = '/tmp'
+
 # Search settings
 OSCAR_SEARCH_SUGGEST_LIMIT = 10
 
@@ -43,4 +51,7 @@ OSCAR_FROM_EMAIL = 'oscar@example.com'
 OSCAR_OFFER_BLACKLIST_PRODUCT = None
 
 # Max total number of items in basket
-OSCAR_MAX_BASKET_QUANTITY_THRESHOLD = None
\ No newline at end of file
+OSCAR_MAX_BASKET_QUANTITY_THRESHOLD = None
+
+#Cookies
+OSCAR_COOKIES_DELETE_ON_LOGOUT = ['oscar_recently_viewed_products', ]
\ No newline at end of file
